// Code door Warrick A Smith en gereproduceerd met vriendelijke toestemming,
// http://startingelectronics.com 26 April 2012
const int latchPin = 8; // Pennen verbonden met schakelregister 74HC595
const int clockPin = 12;
const int dataPin = 11;
const int buttonPin = 2; // Pen verbonden met schakeldraad
// 1 tot 6 en DP (decimale punt) op zeven-segment scherm
unsigned char lookup_7seg[] = {0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x80};
// Patroon van schuddende dobbelsteen op zeven-segment scherm
unsigned char shake_dice[] = {0x63, 0x5C};
// Rollende dobbelsteen op zeven-segment scherm
unsigned char roll_dice[] = {0x1C, 0x58, 0x54, 0x4C};
// Varieer tijdsduur voordat getal van dobbelsteen wordt ontvangen
int rand_seed;
int rand_num = 0; // Genereer willekeurig getal
unsigned char shake_toggle = 0; // voor animatie schuddende dobbelsteen
int index = 0; // Voor animatie rollende dobbelsteen
int shake_speed; // Verhoogt snelheid van schudden dobbelsteen
void setup() {
  pinMode(latchPin, OUTPUT); // Output-pennen voor aansturen van
  // het schakelregister 74HC595
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(buttonPin, INPUT); // Lees status van schakeldraad uit
  digitalWrite(latchPin, LOW); // Toon DP op zeven-segment scherm
  // bij opstarten
  shiftOut(dataPin, clockPin, MSBFIRST, lookup_7seg[6]);
  digitalWrite(latchPin, HIGH);
  randomSeed(analogRead(0)); // Genereer random seed
}
void loop() {
  if (digitalRead(buttonPin)) {
    shake_speed = 150; // Reset snelheid van schudden dobbelsteen
    delay(30);
    // Genereer getal voor willekeurige snelheid en toon animatie schudden
    while (digitalRead(buttonPin)) {
      rand_seed++; // Genereer willekeurig getal
      // Animeer schuddende dobbelsteen
      if (shake_toggle) {
        AnimateDice(0, shake_dice);
        shake_toggle = 0;
      }
      else {
        AnimateDice(1, shake_dice);
        shake_toggle = 1;
      }
      delay(80 + shake_speed); // Verhoog snelheid van animatie
      if (shake_speed > 0) {
        shake_speed -= 10;
      }
    }
    // Animeer rollende dobbelsteen
    for (int rolls = 0; rolls < (rand_seed % 10 + 14); rolls++) {
      AnimateDice(index, roll_dice);
      delay((1 + rolls) * 20);
      index++;
      if (index > 3) {
        index = 0;
      }
    }
    rand_num = random(0, 6); // Genereer gegooid getal op dobbelsteen
    DiceNumber(rand_num);
  }
}
// Toon het getal van dobbelsteen op zeven-segment scherm
void DiceNumber(unsigned char num) {
  digitalWrite(latchPin, LOW);
  shiftOut(dataPin, clockPin, MSBFIRST, lookup_7seg[num]);
  digitalWrite(latchPin, HIGH);
}
// Toon een frame van de schuddende of rollende dobbelsteen
void AnimateDice(int seg, unsigned char *table) {
  digitalWrite(latchPin, LOW);
  shiftOut(dataPin, clockPin, MSBFIRST, table[seg]);
  digitalWrite(latchPin, HIGH);
}
